@@ -17,6 +17,9 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 
+
+# Hannu Väisänen has added some inflection types.
+
 import re
 
 grads = [ (u'sw', u'tt', u'av1'),
@@ -120,6 +123,7 @@ modern_classmap = [(u'valo', u'sw', [(None,u'(.*)',u'valo'),
 			(u't',u'(.*lah)ti',u'lahti'),
 			(u't',u'(.*h)ti',u'lehti'),
 			(u'p',u'(.*)pi',u'siipi'),
+			(u'k>',u'(.*i)ki',u'piki'),
 			(u'k>',u'(.*)ki',u'kaikki')]),
 	(u'nalle', u'sw', [(None,u'(.*Ce)',u'nalle'),
 			 (None,u'(.*Cé)',u'nalle'),
@@ -198,7 +202,8 @@ modern_classmap = [(u'valo', u'sw', [(None,u'(.*)',u'valo'),
 	                  (None,u'(.*r)si',u'hirsi'),
 		        (None,u'(.*l)si',u'jälsi')]),
 	(u'sisar', u'ws', [(None,u'(.*CVC)',u'sisar'),
-		         (u't',u'(.*t)Ar',u'tytär')]),
+		         (u't',u'(.*t)Ar',u'tytär'),
+		         (u'>k',u'(.*i)en',u'ien')]),
 	(u'uistin', u'ws', [(None,u'(.*i)n',u'uistin'),
 	                    (u'nn',u'(.*n)nin',u'vaimennin'),
 		          (u'll',u'(.*l)lin',u'sivellin'),
@@ -230,6 +235,7 @@ modern_classmap = [(u'valo', u'sw', [(None,u'(.*)',u'valo'),
 		          (u'd',u'(.*)dAs',u'ahdas'),
 		          (u'v',u'(.*)vAs',u'varvas'),
 		          (u't',u'(.*t)As',u'ratas'),
+			(u't',u'(.*t)is',u'altis'),
 		          (u'>k',u'(.*)As',u'varas'),
 			(u'>k',u'(.*)is',u'ruis'),
 			(u'>k',u'(.*)es',u'ies')]),
@@ -249,7 +255,8 @@ modern_classmap = [(u'valo', u'sw', [(None,u'(.*)',u'valo'),
 		        (u'v',u'(.*)ve',u'taive'),
 		        (u'k',u'(.*k)e',u'tarvike'),
 		        (u'>k',u'(.*V)e',u'tarvike'),
-		        (u't',u'(.*Vt)e',u'vaate')]),
+		        (u't',u'(.*Vt)e',u'vaate'),
+		        (u't',u'(.*rt)e',u'vaate')]),
 	# Verbs
 	(u'punoa', u'sw', [(None,u'(.*)A',u'punoa'),
 	                   (u'mp',u'(.*m)pUA',u'ampua'),
@@ -274,12 +281,14 @@ modern_classmap = [(u'valo', u'sw', [(None,u'(.*)',u'valo'),
 		         (u'k>',u'(.*)kOA',u'aikoa'),
 		         (u'k>',u'(.*)kUA',u'kiekua')]),
 	(u'aavistaa', u'sw', [(None,u'(.*t)AA',u'aavistaa'),
+	                      (u'rt',u'(.*r)tAA',u'longertaa'),
 	                      (u'tt',u'(.*t)tAA',u'alittaa'),
 			  (u't',u'(.*h)tAA',u'astahtaa')]),
 	(u'hidastaa', u'-', [(None,u'(.*t)AA',u'hidastaa')]),
 	(u'heittää', u'sw', [(u'tt',u'(.*t)tAA',u'heittää')]),
 	(u'muistaa', u'-', [(None,u'(.*C)AA',u'muistaa')]),
-	(u'inttää', u'sw', [(u'tt',u'(.*t)tAA',u'inttää')]),
+	(u'inttää', u'sw', [(u'tt',u'(.*t)tAA',u'inttää'),
+	                    (u't',u'(.*)tAA',u'itää')]),
 	(u'sulaa', u'sw', [(None,u'(.*C)AA',u'sulaa'),
 	                   (u'nt',u'(.*n)tAA',u'kyntää'),
 		         (u'tt',u'(.*t)tAA',u'autioittaa'),
@@ -316,6 +325,8 @@ modern_classmap = [(u'valo', u'sw', [(None,u'(.*)',u'valo'),
 			(u'p',u'(.*)peA',u'rypeä'),
 			(u'k>j',u'(.*)keA',u'polkea'),
 			(u'k>',u'(.*)keA',u'hakea')]),
+	(u'tuntea', u'sw', [(u'nt',u'(.*tUn)teA',u'tuntea')]),
+	(u'lähteä', u'sw', [(u't',u'(.*lA)hteA',u'lähteä')]),
 	(u'sallia', u'sw', [(None,u'(.*C)iA',u'sallia'),
 			(u'nk',u'(.*n)kiA',u'onkia'),
 			(u'mp',u'(.*m)piA',u'empiä'),
@@ -328,9 +339,10 @@ modern_classmap = [(u'valo', u'sw', [(None,u'(.*)',u'valo'),
 			(u'k>j',u'(.*)kiA',u'hylkiä'),
 			(u'k>',u'(.*i)kiA',u'poikia'),
 			(u'k>',u'(.*)kiA',u'loikkia')]),
-	(u'moittia', u'sw', [(u'tt',u'(.*t)tiA',u'moittia')]),
 	(u'voida', u'ws', [(u't',u'(.*)idA',u'voida')]),
 	(u'kanavoida', u'ws', [(u't',u'(.*O)idA',u'kanavoida')]),
+	(u'saada', u'-', [(None,u'(.*CA)AdA',u'saada'),
+	                  (None,u'(.*CU)UdA',u'myydä')]),
 	(u'juoda', u'-', [(None,u'(.*C)UOdA',u'juoda'),
 		        (None,u'(.*C)iedA',u'viedä')]),
 	(u'nuolaista', u'ws', [(None,u'(CAis)tA',u'nousta'),
@@ -377,7 +389,8 @@ modern_classmap = [(u'valo', u'sw', [(None,u'(.*)',u'valo'),
 			(u'll',u'(.*l)litA',u'hellitä'),
 			(u'mm',u'(.*m)mitA',u'lämmitä'),
 			(u'p',u'(.*p)UtA',u'silputa'),
-			(u'v',u'(.*)vUtA',u'vivuta')]),
+			(u'v',u'(.*)vUtA',u'vivuta'),
+			(u'>k',u'(.*)itA',u'keritä')]),
 	(u'juoruta', u'ws', [(None,u'(.*U)tA',u'juoruta'),
 			(u'mm',u'(.*m)mUtA',u'kummuta'),
 			(u't',u'(.*t)UtA',u'luututa'),
