@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 
-# Copyright 2007 Hannu Väisänen (Etunimi.Sukunimi@joensuu.fi
+# Copyright 2007-2008 Hannu Väisänen (Etunimi.Sukunimi@joensuu.fi
 # Program to generate lexicon files for Suomi-malaga Sukija edition.
 
 # This program is free software; you can redistribute it and/or modify
@@ -27,21 +27,47 @@ import re
 import sys
 
 # Historical inflections in alphabetical order.
-historical = [(u'ahven', u'ws', [(None,u'(.*CVC)',u'ahven')]),
+historical = [(u'aavistaa', u'sw', [(u'tt',u'(.*O)ittAA',u'kirjoittaa'),
+				    (u'tt',u'(.*O)ttAA',u'ammottaa'),
+				    (None,u'(.*t)AA',u'aavistaa'),
+				    (u'tt',u'(.*[AeiU]t)tAA',u'alittaa'),
+				    (u't',u'(.*h)tAA',u'astahtaa')]),
+	      (u'arvailla',   u'-',  [(None,u'(.*[AOU]])illA',u'arvailla')]),
+	      (u'hohtaa',     u'sw', [(u'tt',u'(.*t)tAA',u'heittää')]),
+	      (u'iäkäs',      u'ws', [(u'k',u'(.*k)As',u'iäkäs')]),
+	      (u'katsella',   u'ws', [(None,u'(.*[AOU])illA',u'arvailla')]),
+	      (u'kaivaa',     u'sw', [(None,u'(.*aj)AA',u'ajaa')]),
+	      (u'kirjoittaa', u'sw', [(u'tt',u'(.*O)ittAA',u'kirjoittaa'),
+				      (u'tt',u'(.*O)ttAA',u'ammottaa'),
+				      (u'tt',u'(.*[AeiU]t)tAA',u'asettaa')]),
+	      (u'kuollut', u'-', [(None,u'(.*neits)yt',u'neitsyt'),
+				  (None,u'(.*C)lUt',u'kuollut'),
+				  (None,u'(.*)nUt', u'punonut'),
+				  (None,u'(.*C)rUt',u'purrut'),
+				  (None,u'(.*C)sUt',u'juossut')]),
+	      (u'onneton', u'ws', [(None,u'(.*)tOn',u'alaston'),
+				   (u't',u'(.*)tOn',u'onneton')]),
+	      (u'punoa',   u'sw', [(u't',u'(...*AU)tUA',u'antautua')]),
+##	      (u'vieras',  u'ws', [(u'k',u'(.*k)As',u'iäkäs'),
+##				   (None,u'(.*[lr]iA)s',u'utelias'),
+##				   (u'>k',u'(.*)As',u'vilkas')]),
+	(u'ahven',    u'ws', [(None,u'(.*CVC)',u'ahven')]),
         (u'antautua', u'sw', [(u't',u'(.*)tUA',u'antautua')]),
-        (u'banaali', u'sw', [(None,u'(.*)i',u'banaali'),
-                        (u'nt',u'(.*n)ti',u'hollanti'),
-                        (u'nk',u'(.*n)ki',u'killinki'),
-                        (u'kk',u'(.*k)ki',u'kajakki'),
-                        (u'tt',u'(.*t)ti',u'salaatti'),
-                        (u'pp',u'(.*p)pi',u'sinappi'),
-                        (u't',u'(.*)ti',u'konvehti') ]),
+	(u'altis',    u'ws', [(None, u'(.*t)is', u'altis')]),
+        (u'banaali',  u'sw', [(None,u'(.*)i',u'banaali'),
+                              (u'nt',u'(.*n)ti',u'hollanti'),
+			      (u'nk',u'(.*n)ki',u'killinki'),
+			      (u'kk',u'(.*k)ki',u'kajakki'),
+			      (u'tt',u'(.*t)ti',u'salaatti'),
+			      (u'pp',u'(.*p)pi',u'sinappi'),
+			      (u't',u'(.*)ti',u'konvehti') ]),
         (u'bébé', u'-', [(None,u'(.*V)',u'bébé')]),
         (u'haastaa', u'sw', [(None,u'(.*C)AA',u'haastaa')]),
         (u'kaihtaa', u'sw', [(u't',u'(.*)tAA',u'kaihtaa')]),
         (u'kantaja', u'-', [(None,u'(.*)jA',u'johdin_jA_kantaja')]),
         (u'kirjoitella', u'ws', [(None,u'(...*O)itellA',u'kilvoitella'),
                                  (None,u'(.*O)tellA',u'ilotella')]),
+	(u'tulla', u'ws', [(None,u'(.*Vl)lA',u'tulla')]),
         (u'koiras', u'ws', [(None,u'(.*A)s',u'koiras')]),
 	(u'kutiaa', u'-',  [(None,u'(.*Cia)a',u'kutiaa')]),
         (u'laittaa', u'sw', [(u'tt',u'(.*t)tAA',u'laittaa')]),
@@ -57,13 +83,14 @@ historical = [(u'ahven', u'ws', [(None,u'(.*CVC)',u'ahven')]),
         (u'taittaa', u'sw', [(u'tt',u'(.*t)tAA',u'taittaa')]),
         (u'tuomi', u'-', [(None,u'(.*V)mi',u'tuomi')]),
 	(u'uros', u'-', [(None,u'(.*)s',u'uros')]),
+	(u'valmis',u'ws', [(None,u'(.*)is',u'valmis')]),
 	(u'veranta', u'sw', [(u'nt',u'(.*n)tA',u'veranta')]),
 	(u'vihanta', u'sw', [(u'nt',u'(.*n)tA',u'vihanta')]),
         (u'virkkaa', u'sw', [(u'kk',u'(.*k)kAA',u'jakaa')])
         ]
 
-classmap = hfconv.modern_classmap
-classmap.extend(historical)
+classmap = historical
+classmap.extend(hfconv.modern_classmap)
 
 pattern = u"^(?P<alku>.*)(?:" + \
           u"(?P<ammottaa>OttAA)|" + \
@@ -98,6 +125,13 @@ pattern = pattern.replace(u"U", u"[uy]")
 pattern = pattern.replace(u"C", u"[bcdfghjklmnpqrstvwxzšžçðñþß]")
 rx = re.compile(pattern, re.IGNORECASE)
 
+begin = u"(amerikan|jälleen|tiibetin|uudelleen).+"
+rx_begin = re.compile(begin, re.IGNORECASE)
+
+end = u".+(herkkä|pöllö|rämeä|valmis)"
+rx_end = re.compile(end, re.IGNORECASE)
+
+
 #print pattern
 
 # Sanat, jotka tunnistetaan Sukija-versiossa automaagisesti toisten
@@ -112,31 +146,60 @@ rx = re.compile(pattern, re.IGNORECASE)
 # grep '<form>' ../*/*xml | sed -e "s@</\?form>@@g" | sort
 #
 #        Sukija-versiossa "amerikan" on etuliite. (-:
-words = [u"aikalainen", u"alaisuus",
+#u"punanen", u"päivänen",
+words = [
+	 u"aikalainen", u"alaisuus", u"alastulo",
+	 u"allekirjoittaa", u"alleviivata",
 	 u"amerikanenglanti", u"amerikanenglantilainen", u"amerikanjuutalainen",
          u"amerikanrauta", u"amerikansalaatti", u"amerikansuomalainen", u"amerikansuomi",
-         u"arastella",
-         u"elämä", u"etruskilainen",
+         u"ammattilainen", u"arastella",
+         u"edellisvuotinen", u"elämä", u"esilletuominen", u"esilletuonti",
+	 u"esimerkki", u"etruskilainen",
          u"freudilainen",
-	 u"halpatyö", u"hautaus", u"herraskartano", u"herrasmies", u"herraspoika", u"herrasväki",
-	 u"institutionalisoitua", u"itkettynyt", u"itkettyä",
+	 u"halpatyö", u"hautaus", u"herraskartano", u"herrasmies", u"herraspoika",
+	 u"herrasväki",
+	 u'huiskauttaa', u"hyvinvoida", u"hyvinvointi",
+	 u"innoittua", u"institutionalisoitua", u"itkettynyt", u"itkettyä",
+	 u"julkishallinto", u"julkissektori", u"julkistalous",
 	 u"jumalaistaru", u"jumalaistarusto", u"jälkeenjäänyt",
-	 u"kansalainen", u"kansallismielinen", u"kuntalainen", u"kyyhkyläinen", u"käynti",
-	 u"laajakangas", u"laajakulma", u"laajakuva", u"lahkolainen", u"lahopuu",
-	 u"lahtelainen", u"liittolainen", u"luokkalainen", u"lyönti",
-	 u"maallistua", u"maolainen", u"marxilainen", u"mäkitupalainen",
-	 u"oikeistolainen", u"oikeus", u"opetus", u"otto",
-         u"pakolainen", u"pito",
-	 u"rivittyä",
-	 u"sisäänajo", u"sisäänmeno", u"sisääntulo",
+	 u"kansalainen", u"kansallismielinen", u"kestitys", u"kumous", u"kuntalainen",
+	 u"kuolleenkoura", u"kuolleenmiehenkytkin",
+	 u"kurssilainen", u"kymmenvuotinen", u"kyyhkyläinen",
+	 u"käsinkylvö", u"käsinpesu", u"käsinlypsy", u"käsinkohonta",
+	 u"käsinladonta", u"käsinlajittelu", u"käsinlatoja", u"käsinohjaus",
+	 u"käsinseisonta", u"käsinkäynti",
+	 u"käynti", u"käytävä",
+	 u"laajakaista", u"laajakangas", u"laajakulma", u"laajakuva",
+	 u"laakafilmi", u"laakalyönti", u"laakapaino", u"laakapihti",
+	 u"lahkolainen", u"lahopuu",
+	 u"lahtelainen", u"laulattaa", u"leniniläinen", u"liittolainen", u"lojaalisuus",
+	 u"luokkalainen", u"luontoistalous", u"lyhytvalinta", u"lyönti",
+	 u"maailmanympärimatka",
+	 u"maallistua", u"maattojohdin", u"maattojohto", u"maolainen",
+	 u"marxilainen", u"marksismi-leninismi", u"marxismi-leninismi",
+	 u"metsäläinen", u"mierolainen",
+	 u"mikkelinpäivä", u"mittatarkka", u"mongolilainen", u"mäkitupalainen",
+	 u"naarmuttua", u"nimismies",
+	 u"ohutsuoli", u"oikeanlainen", u"oikeistolainen", u"oikeus",
+	 u"omanlainen", u"opetus", u"otto",
+         u"pakolainen", u"peräänajo", u"pito", u"poltto",
+	 u"rivittyä", u"romahdusmainen", u"ruskettua", u"rusketus",
+	 u"siirtolainen", u"sisäpiiriläinen", u"sisäänajo", u"sisäänmeno",
+	 u"sisääntulo", u"strippaus", u"säteisliike", u"sätkytellä",
+	 u"takaisinkytkentä",
 	 u"tietynlainen", u"trotskilainen", u"tuonti", u"tupalainen", u"työläinen",
 	 u"täysihoito", u"täysihoitola",
-	 u"vajaamielinen", u"vasemmistolainen", u"voima", u"wrightiläinen", u"väärinkäsitys",
+	 u"uloskäytävä", u"uurteisvalas",
+	 u"vaivaisenluu", u"vajaamielinen", u"valmistuttaa",
+	 u"valmisateria", u"valmisbetoni", u"valmismatka", u"valmisosa",
+	 u"valmispakkaus", u"valmisruoka", u"valmistalo", u"valmistuote", u"valmisvaate",
+	 u"vasemmistolainen", u"vastaavanlainen", u"veto",
+	 u"voima", u"wrightiläinen", u"väärinkäsitys",
 	 u"vähittäishinta", u"vähittäiskauppa", u"vähittäismaksu", u"vähittäismyymälä",
          u"vähittäismyynti",
-	 u"ylivoimainen", u"ylösnoussut", u"yritellä", u"ystävällismielinen",
+	 u"ylösnoussut", u"yritellä", u"ystävällismielinen",
 	 u"Aaltonen", u"Aamu", u"Aarnio", u"Aho", u"Alanko", u"Alkio", u"Arabia",
-	 u"Armenia", u"Aro", u"Asukas", u"Aura", u"Aurinko", u"Autio",
+	 u"Armenia", u"Aro", u"Astuvansalmi", u"Asukas", u"Aura", u"Aurinko", u"Autio",
 	 u"Bulgaria",
 	 u"Eesti", u"Elo", u"Englanti", u"Eno", u"Espanja", u"Esteri",
 	 u"Georgia", u"Guinea",
@@ -146,28 +209,28 @@ words = [u"aikalainen", u"alaisuus",
 	 u"Kallio", u"Kangas", u"Kannus", u"Kanttura", u"Karjala", u"Karjalainen", u"Kari",
          u"Kartano", u"Kartio", u"Kaste",
 	 u"Kerttu", u"Kerttunen", u"Kielo", u"Kiina", u"Kilpi", u"Kisko", u"Knuutinpoika",
-         u"Koitto", u"Koivisto", u"Koivu", u"Kolari", u"Kotka", u"Koppelo",
+         u"Koitto", u"Koivisto", u"Koivu", u"Kokkari", u"Kolari", u"Kotka", u"Koppelo",
 	 u"Koski", u"Kreikka", u"Kroatia", u"Kuha", u"Kuikka", u"Kukka", u"Kulho",
-	 u"Kuokka", u"Kurki", u"Kurppa", u"Kuru", u"Kuutti", u"Kytö",
-	 u"Laakso", u"Lahja", u"Lahti", u"Laina", u"Laine", u"Lauha",
+	 u"Kuokka", u"Kurki", u"Kurppa", u"Kuru", u"Kuutti", u"Kytö", u"Kärkistensalmi",
+	 u"Laakso", u"Lahja", u"Lahti", u"Laina", u"Laine", u"Lappi", u"Lauha",
 	 u"Lehto", u"Lemu", u"Lilja", u"Lintunen", u"Liperi", u"Lotta",
-	 u"Maa", u"Maila", u"Malmi", u"Marja", u"Matti", u"Mela", u"Metso", u"Miina",
-	 u"Minttu", u"Missi", u"Moisio", u"Muikku", "Murto",
+	 u"Maa", u"Maila", u"Malmi", u"Marja", u"Martta", u"Matti", u"Mela", u"Metso", u"Miina",
+	 u"Minttu", u"Missi", u"Moisio", u"Muikku", u"Murto", u"Mäkinen",
 	 u"Niemi", u"Niva", u"Norja",
 	 u"Oiva", u"Orvokki", u"Otto",
 	 u"Pajari", u"Palmu", u"Paula", u"Perho", u"Pinja",
 	 u"Pohja", u"Pokka", u"Portugali", u"Posti", u"Puola", u"Päivä",
 	 u"Raitio", u"Ranta", u"Ranska", u"Rauha", u"Rautio", u"Rautu", u"Reima", u"Riski",
 	 u"Ruotsi", u"Ruukki", u"Ruusu", u"Rönkkö",
-	 u"Saari", u"Saksa", u"Salo", u"Satu", u"Savikko", u"Seppä",
+	 u"Saari", u"Saksa", u"Saku", u"Salo", u"Satu", u"Savikko", u"Seppä",
 	 u"Siimes", u"Sikiö", u"Sini", u"Sirkka", u"Sisko",
 	 u"Somero", u"Sorsa", u"Sulo", u"Suomi", u"Säde", u"Säteri",
 	 u"Taimi", u"Taisto", u"Tallus", u"Tanska", u"Tapani", u"Tarkka", u"Tarmo",
-	 u"Terho", u"Terttu", u"Tikka", u"Timpuri", u"Toivo",
+	 u"Terho", u"Terttu", u"Tiibet", u"Tikka", u"Timpuri", u"Toivo",
          u"Tšekki", u"Tuisku", u"Turkki", u"Tuuli",
 	 u"Ukraina", u"Unelma", u"Unkari",
 	 u"Valta", u"Valtti", u"Varis", u"Varkaus", u"Varma", u"Vasara", u"Vento",
-         u"Venäjä", u"Venäläinen", u"Vesa", u"Vieno", u"Viita", u"Viro", u"Virolainen",
+         u"Venäjä", u"Venäläinen", u"Vesa", u"Vieno", u"Vietnam", u"Viita", u"Viro", u"Virolainen",
 	 u"Virta", u"Visa", u"Vuokko", u"Vuoksi",
 	 u"Ylämaa"]
 
@@ -408,13 +471,17 @@ def handle_word(main_vocabulary,vocabulary_files,word):
 							u"#Malaga class not found for (%s, %s)\n" \
 			                   % (wordform, voikko_infclass))
 			continue
-		
 		if (wordform in words):
-#			print ("Ei tarvita: " + wordform + u"\n")
+#			print ("Ei tarvita: " + wordform)
 			continue
-		if (((wordform == u"neitsyt") and (jatko == u"airut")) or
-		    ((wordform == u"ori")     and (jatko == u"risti"))):
-#			print ("Ei tarvita: " + wordform + u"\n")
+		if (rx_begin.match(wordform) != None):
+#			print ("Ei tarvita: " + wordform)
+			continue
+		if (rx_end.match(wordform) != None):
+#			print ("Ei tarvita: " + wordform)
+			continue
+		if ((wordform == u"ori") and (jatko == u"risti")):
+#			print ("Ei tarvita: " + wordform)
 			continue
 		
 		nsyl = number_of_syllabels(wordform)
@@ -453,134 +520,30 @@ def handle_word(main_vocabulary,vocabulary_files,word):
 		if ((jatko == u"nainen") and (nsyl == 3) and
 		    (malaga_word_class in [u"nimisana", u"nimi_laatusana", u"laatusana"]) and
 		    (rx != None) and (d != None) and (d['keltainen'] != None) and
-		    (not (wordform in [u"armainen", u"omainen"]))):
+		    (not (wordform in [u"armainen", u"lukunen", u"omainen", u"osanen", u"vastuinen"]))):
 			if (wordform[-4] == u"i"):
 				alku2 = alku[:-1]    # Keltainen => keltanen.
 			else:
 				alku2 = alku + u"i"  # Hevonen => hevoinen.
 			jatko2 = jatko
 			wordform2 = alku2 + u"nen"
-#			print (u"Keltainen " + wordform2 + u" " + alku2 + u" " + jatko2)
-		#
-		# Kolmitavuiset, perusmuodossaan O(i)ttAA-loppuiset sanat
-		# tunnistetaan Sukija-versiossa sekä i:n kanssa että ilman.
-		#
-#		elif ((jatko == u"alittaa") and (nsyl == 3) and
-#		    (rx != None) and (d != None) and ((d['ammottaa'] != None) or (d['kirjoittaa'] != None))):
-#			jatko = u"kirjoittaa"
-#			jatko2 = u"kirjoittaa"
-#			if (d['kirjoittaa'] != None):
-#				alku2 = wordform[0:-5] + u"t"   # Kirjoittaa => kirjottaa.
-#			elif (d['ammottaa'] != None):
-#				alku2 = wordform[0:-4] + u"it"  # Ammottaa => ammoittaa.
-#			else:
-#				print(u"Wordform=" + wordform + u"\n")
-#				abort()
-#			wordform2 = alku2 + wordform[-3:]
-#			print ("Aa1 " + s + u" w=" + wordform2 + u" a=" + alku2 + u" A=" + d['alku'] + u"\n")
-#		#
-#		# Muutetaan taivutus ammottaa tai kirjoittaa => alittaa tai kirjoittaa.
-#		#
-#		elif ((jatko in [u"ammottaa", u"kirjoittaa"]) and (nsyl > 2)):
-#			if ((d != None) and (d['ammottaa'] != None)):
-#				jatko = u"kirjoittaa"
-#				jatko2 = jatko
-#				alku = wordform[0:-4] + u"t"
-#				alku2 = wordform[0:-4] + u"it"
-#				wordform2 = alku2 + wordform[-3:]
-##				print ("Aa3 " + s + u" " + wordform2 + u" " + alku2 + u"\n")
-#			elif ((d != None) and (d['kirjoittaa'] != None)):
-#				jatko = u"kirjoittaa"
-#				jatko2 = jatko
-#				alku = wordform[0:-5] + u"it"
-#				alku2 = wordform[0:-5] + u"t"
-#				wordform2 = alku2 + wordform[-3:]
-##				print ("Aa4 " + s + u" " + wordform2 + u" " + alku2 + u"\n")
-#			elif (jatko == u"ammottaa"):
-#				jatko = u"kirjoittaa"
-#				alku = alku + u"t"
-#			else:
-#				jatko = u"alittaa"
-#				alku = alku + u"t"
-		#
-		# Muutetaan taivutuksia: alittaa => ammottaa tai kirjoittaa; ammottaa => alittaa.
-		#
-#		elif ((jatko in [u"alittaa"]) and (nsyl == 3) and
-#		      (rx != None) and (d != None) and ((d['ammottaa'] != None) or (d['kirjoittaa'] != None))):
-#			print (u"a1 " + wordform + u" " + alku + u" " + jatko)
-#			if (alku[-2:] == u"it"):
-#				alku = alku[0:-2]
-#			else:
-#				alku = alku[0:-1]
-#			if (d['ammottaa'] != None):
-#				jatko = u"ammottaa"
-#				print (u"a2 " + wordform + u" " + alku + u" " + jatko)
-#			elif (d['kirjoittaa'] != None):
-#				jatko = u"kirjoittaa"
-#				print (u"a2 " + wordform + u" " + alku + u" " + jatko)
-#		elif ((jatko in [u"ammottaa", u"kirjoittaa"]) and (nsyl > 2) and (rx != None)):
-#			print (u"b1 " + wordform + u" " + alku + u" " + jatko + "\n")
-#			if ((d != None) and ((d['ammottaa'] != None) or (d['kirjoittaa'] != None))):
-#				if (alku[:-1] == u"i"):
-#					alku = alku[0:-1]
-#					print (u"b2 " + wordform + u" " + alku + u" " + jatko)
-#			else:
-#				alku = alku + u"t"
-#				jatko = u"alittaa"
-#				print (u"b2 " + wordform + u" " + alku + u" " + jatko)
-		#
-		# Kolmitavuiset nuolaista-tyyppiset sanat
-		# tunnistetaan Sukija-versiossa sekä i:n kanssa että ilman.
-		#
-		elif ((jatko == u"nuolaista") and (nsyl == 3)):
-			if ((d != None) and (d['nuolaista'] != None)):
-				alku = wordform[0:-4] + u"is"
-				alku2 = wordform[0:-4] + u"s"
-				jatko2 = jatko
-				wordform2 = alku2 + wordform[-2:]
-#				print("N1 " + wordform2 + u" " + alku2 + u" " + s + u"\n")
-			else:
-				alku2 = alku + u"s"
-				jatko2 = jatko
-				alku = alku + u"is"
-				wordform2 = alku2 + wordform[-2:]
-#				print("N2 " + wordform2 + u" " + alku2 + u" " + s + u"\n")
-		elif ((nsyl == 4) and (rx != None) and (d != None) and (d['antautua'] != None) and (jatko != u"antautua")):
-			jatko = u"antautua"
-#			print (u"Antautua " + wordform);
-		elif ((wordform == u"ajaa") and (jatko == u"kaivaa")):
-			jatko = u"ajaa"
+##			if (wordform[-4] == u"i"):
+##				alku = wordform[:-4]    # Keltainen => keltanen.
+##				jatko = u"punainen"
+##			else:
+##				jatko = u"hevoinen"
+##			print (u"Keltainen " + wordform + u" " + alku + u" " + jatko)
 		elif (jatko in [u"asiakas", u"avokas"]):
 			jatko = u"iäkäs"
 		elif (jatko == u"varas"):
 			jatko = u"vilkas"
 		elif ((jatko == u"vieras") and (d != None) and (d['utelias_ankerias'] != None)):
 			jatko = u"utelias"
-		elif (jatko == u"jättää"):
-			jatko = u"heittää"
-		elif (wordform == u"piillä"):
-			jatko = u"nuolla"
 		#
 		# Korjataan alku- ja jatko-kenttien arvoja.
 		#
-		elif ((nsyl > 2) and (jatko == u"arvailla")):
-			alku = alku[:-2]
 		elif (jatko == u"rakentaa"):
 			alku = wordform[:-4]
-		elif (jatko in [u"onneton", u"alaston"]):
-			alku = alku[:-1]
-		elif ((jatko == u"kuollut") and (wordform == u"neitsyt")):
-			jatko = u"neitsyt"
-		elif ((jatko == u"ori") and (wordform == u"ori")):
-			alku = u"or"
-		elif (jatko == u"kuollut"):
-			if (wordform[-3] == u"n"):
-				jatko = u"punonut"
-			elif (wordform[-3] == u"r"):
-				jatko = u"purrut"
-			elif (wordform[-3] == u"s"):
-				jatko = u"juossut"
-			alku = alku[:-1]
 		elif ((nsyl > 2) and (jatko == u"vastaus") and
 		      (rx != None) and (d != None) and ((d['aivoitus'] != None) or (d['jaotus'] != None))):
 			#
